/*
    Spring-Mass Simulation
    (C) 2007 Jason Hunt
    nulluser@gmail.com
*/

#include <windows.h>
#include <stdio.h>
#include <math.h>

#include "time.h"

#define SECOND_TIMER    1           // ID for frame timer
#define UPDATE_TIMER    2           // ID for frame timer

#define FRAME_TIME      10          // Millisecond delay for frames

#define max_points_x    400
#define max_points_y    400

/* Point type */
typedef struct point2d
{
    double x;
    double y;
};


/* Tension */
typedef struct tension_type
{
    double mag;
    double angle;
};


/* Point in sheet */
typedef struct point_type
{
    double x, y;        // Pos
    double vx, vy;      // Vel
    double ax, ay;      // Accl
};

int display_x = 640;             // Initial screen size
int display_y = 600;


double inital_mouse_x;
double inital_mouse_y;
double mouse_time;

bool mouse_down = false;


HDC memdc = NULL;	           	 // Memory device context
HBITMAP h_mem_bitmap = NULL;     // Offscreen bitmap

bool apply_grav = true;
bool force_enable = false;

unsigned int fps = 0;
unsigned int updates = 0;

int test_pt_x = 0;
int test_pt_y = 0;

// Parameters

double max_x = 10, max_y = 10;


double sheet_x_size = 6.0;
double sheet_y_size = 6.0;


double sheet_x_pos = max_x / 2 - sheet_x_size/2;
double sheet_y_pos = 0;


int points_x = 5;
int points_y = 5;

double point_size_x = 0.1 * sheet_x_size / (double)points_x;
double point_size_y = 0.1 * sheet_y_size / (double)points_y;


//double drag = 1.0;
double drag = 3.6;

double sheet_mass = 1.0;    // Mass of entire sheet
double base_spring_k = 1000.0;


double base_grav = 10;

double point_mass = sheet_mass / (double)(points_x * points_y);
double spring_k = base_spring_k /  (double)(points_x * points_y);
double grav = base_grav * point_mass;

double force_freq = 0.1;


double rest_len_norm;
double rest_len_diag;


// Data set

point_type points[max_points_y][max_points_x];

tension_type vert_tensions[max_points_y+1][max_points_x+1];
tension_type horz_tensions[max_points_y+1][max_points_x+1];

tension_type diag1_tensions[max_points_y+1][max_points_x+1];
tension_type diag2_tensions[max_points_y+1][max_points_x+1];




/* Return a random number in the specified range */
double f_rand(double s, double e)
{
    return((e - s)*(rand() / (double)RAND_MAX) + s);
}
/* End of f_rand */



/* Update simulation constants */
void update_const( void )
{
    point_mass = sheet_mass / (double)(points_x * points_y);
    spring_k = base_spring_k / (double)(points_x * points_y);    

    grav = base_grav * point_mass;

    point_size_x = 0.1 * sheet_x_size / (double)points_x;
    point_size_y = 0.1 * sheet_y_size / (double)points_y;
    
    // Get average rest dist
    rest_len_norm = (sheet_x_size / (double)(points_x + 1) + sheet_y_size / (double)(points_y + 1))/2.0;

    // the diagnoals need a differnt rest dist
    rest_len_diag = sqrt(2.0) * rest_len_norm;
}
/* End of update_const */


/* Setup base sheet */
void init_sheet( void )
{
    update_const();
    
    for (int y = 0; y < points_y; y++)
        for (int x = 0; x < points_x; x++)
        {
             points[y][x].x = ((x+1) / (double)(points_x + 1)) * sheet_x_size + sheet_x_pos;
             points[y][x].y = ((y+1) / (double)(points_y + 1)) * sheet_y_size + sheet_y_pos;
                
            // Normalize modifier by point density
            double rel_size_x = sheet_x_size / (double)points_x;
            double rel_size_y = sheet_y_size / (double)points_y;
        
            // Position varience
           // points[y][x].x += f_rand(-0.2*rel_size_x, 0.2*rel_size_x);
           // points[y][x].y += f_rand(-0.2*rel_size_y, 0.2*rel_size_y);

            // velocity varience                
            points[y][x].vx = 0;//f_rand(-0.20*rel_size_x, 0.20*rel_size_x);
            points[y][x].vy = 0;//f_rand(-0.20*rel_size_y, 0.20*rel_size_y);

            points[y][x].ax = 0;
            points[y][x].ay = 0;
        }
}
/* End iof init_sheet */


/* Turn two cords into a point */
inline point2d get_point( double x, double y)
{
    point2d p;
    p.x = x; p.y=y;
    return(p);
}
/* End of get point */


/* get force from delta and rest length */
inline double get_force(point2d delta, double rest)
{
    double disp = sqrt(delta.x*delta.x + delta.y*delta.y) - rest;
    
    // Normal hooks law, F=-kx
    return(  -spring_k * disp);  
    

    // Super springs and otehr suck madness

    double dir = disp > 0 ? 1.0 : -1.0;  
    double force = -spring_k * disp;
  
   // if (fabs(force) > 10.0)
   //     force = -dir * 10.0;
    
//    return( force );     

   // return(- dir * spring_k * log(fabs(disp)+1));

   // if (disp < 1) disp = 1;
//    return( -spring_k * pow(disp,0.0001));

}


/* Get and angle from a delta */
inline double get_angle(point2d delta)
{
    return( atan2(delta.y, delta.x) );       // Normal hooks law, F=-kx
}


/* Get a tension from a delta and a relaxed length */
inline tension_type get_tension(point2d delta, double base)
{
    tension_type t;

    t.mag = get_force(delta, base);
    t.angle= get_angle(delta);

    return (t);
}
/* End of get tension */


/* Calculate all tensions */
void calc_tension(void)
{
    // calulate vert tensions

    point2d delta;

    for (int ix = 0; ix < points_x; ix++)
    {
        // top of sheet to first point

        double base_x = (((ix+1) / (double)(points_x + 1)) * sheet_x_size + sheet_x_pos);         

        delta.x = points[0][ix].x - base_x;
        delta.y = points[0][ix].y - sheet_y_pos;
    
        vert_tensions[0][ix] = get_tension(delta, rest_len_norm);
     
        // All springs below              
        for (int iy = 1; iy < points_y; iy++)
        {
             delta.x = points[iy][ix].x - points[iy-1][ix].x;
             delta.y = points[iy][ix].y - points[iy-1][ix].y;

             vert_tensions[iy][ix] = get_tension(delta, rest_len_norm);
        }
        
        // Pottom point in column to bottom of frame        
        delta.x = base_x - points[points_y-1][ix].x;
        delta.y = sheet_y_pos + sheet_y_size - points[points_y-1][ix].y;

        vert_tensions[points_y][ix] = get_tension(delta, rest_len_norm);           
    }

    // Calculate horz tensions
    for (int iy = 0; iy < points_y; iy++)
    {
        // Left of sheet to first point

        double base_y = (((iy+1) / (double)(points_y + 1)) * sheet_y_size + sheet_y_pos);         

        delta.x = points[iy][0].x - sheet_x_pos;
        delta.y = points[iy][0].y - base_y;
        
        horz_tensions[iy][0] =  get_tension(delta, rest_len_norm);


        for (int ix = 1; ix < points_x; ix++)
        {
            delta.x = points[iy][ix].x - points[iy][ix-1].x;
            delta.y = points[iy][ix].y - points[iy][ix-1].y;

            horz_tensions[iy][ix] =  get_tension(delta, rest_len_norm);
        }
        
        // bottom point in column to bottom of frame
        
        delta.x = sheet_x_pos + sheet_x_size - points[iy][points_x-1].x;
        delta.y = base_y - points[iy][points_x-1].y;   

        horz_tensions[iy][points_x] = get_tension(delta, rest_len_norm);
    }

    // Upper left to lower right diag
    for (int ix = 0; ix < points_x-1; ix++)
    {
        // All springs below              
        for (int iy = 0; iy < points_y-1; iy++)
        {
             delta.x = points[iy+1][ix+1].x - points[iy][ix].x;
             delta.y = points[iy+1][ix+1].y - points[iy][ix].y;

             diag1_tensions[iy][ix] = get_tension(delta, rest_len_diag);
        }
    }

    // Lower left to ipper right diag
    for (int ix = 0; ix < points_x-1; ix++)
    {
        // All springs below              
        for (int iy = 0; iy < points_y-1; iy++)
        {
             delta.x = points[iy][ix+1].x - points[iy+1][ix].x;
             delta.y = points[iy][ix+1].y - points[iy+1][ix].y;

             diag2_tensions[iy][ix] = get_tension(delta, rest_len_diag);
        }
    }
}



/* Clip a point into the display area */
inline void clip( point_type &p )
{
    if (p.x < point_size_x/2.0)
    {
        p.x = point_size_x/2.0 ;//- (points[iy][ix].y - m);
        p.vx *=  -1;
     }

     if (p.y < point_size_y/2.0)
     {
         p.y = point_size_y/2.0;//- (points[iy][ix].y - m);
         p.vy *=  -1;
     }

     if (p.x > max_x - point_size_x/2.0)
     {
        p.x = max_x-point_size_x/2.0 ;//- (points[iy][ix].y - m);
        p.vx *=  -1;
    }

    if (p.y > max_y-point_size_y/2.0)
    {
        p.y = max_y -point_size_y/2.0;//- (points[iy][ix].y - m);
        p.vy *=  -1;
    }
    
}



/* Move all points */

void update_points( HWND hwnd )
{
    static double last_time = get_time();
   
    double time = get_time();
    double time_diff = time - last_time;
  
    last_time = time;  
  
  
    for (int iy = 0; iy < points_y; iy++)
    {
        for (int ix = 0; ix < points_x; ix++)
        {
            double t1 = horz_tensions[iy][ix].mag; 
            double t2 = horz_tensions[iy][ix+1].mag;
            double t3 = vert_tensions[iy][ix].mag;
            double t4 = vert_tensions[iy+1][ix].mag;
  
            double t5=0, t6=0, t7=0, t8=0;
            double angle5, angle6, angle7, angle8;

            // no horz and vert tensions on edges
            if (ix == 0) t1 = 0;
            if (ix == points_x - 1) t2 = 0;

            if (iy == 0) t3 = 0;
            if (iy == points_y - 1) t4 = 0;
            
                        
            if (ix != 0 && iy != 0)
            {
                t5 = diag1_tensions[iy-1][ix-1].mag;
                angle5 = diag1_tensions[iy-1][ix-1].angle;
            }

            if (ix > 0 && iy < points_y-1)    
            {
                t7 = diag2_tensions[iy][ix-1].mag;
                angle7 = diag2_tensions[iy][ix-1].angle;
            }
  
            if (ix != points_x-1 && iy != points_y -1)
            {
                t6 = diag1_tensions[iy][ix].mag;
                angle6 = diag1_tensions[iy][ix].angle;
            }

            if (iy > 0 && ix < points_x-1) 
            {
                t8 = diag2_tensions[iy-1][ix].mag;
                angle8 = diag2_tensions[iy-1][ix].angle;

            }
    
            double angle1 = horz_tensions[iy][ix].angle;
            double angle2 = horz_tensions[iy][ix+1].angle;

            double angle3 = vert_tensions[iy][ix].angle;
            double angle4 = vert_tensions[iy+1][ix].angle;



            double fx = 0;
            double fy = 0;
        
            if (apply_grav) fy += grav; // Apply gravity
           
            fx += t1 * cos(angle1) - t2 * cos(angle2);  // horx for left and right springs
            fy += t1 * sin(angle1) - t2 * sin(angle2);  // vert for left and right springs
    
            fx += t3 * cos(angle3) - t4 * cos(angle4);  // horx for up and down springs
            fy += t3 * sin(angle3) - t4 * sin(angle4);  // vert for up and down springs
    
            fx += t5 * cos(angle5) - t6 * cos(angle6);  // horx for up and down springs
            fy += t5 * sin(angle5) - t6 * sin(angle6);  // vert for up and down springs
    
            fx += t7 * cos(angle7) - t8 * cos(angle8);  // horx for up and down springs
            fy += t7 * sin(angle7) - t8 * sin(angle8);  // vert for up and down springs
    
    
    time_diff = 0.01;
    
            // Do not calculate new data for point being moved
            if (!(mouse_down && ix == test_pt_x && iy == test_pt_y))
            {
            //    points[iy][ix].ax = fx / point_mass - 50.5 * points[iy][ix].vx;
//                points[iy][ix].ay = fy / point_mass - 50.5* points[iy][ix].vy;


                points[iy][ix].ax = fx / point_mass - drag * points[iy][ix].vx;
               points[iy][ix].ay = fy / point_mass - drag * points[iy][ix].vy;

                points[iy][ix].vx += time_diff * points[iy][ix].ax;
                points[iy][ix].vy += time_diff * points[iy][ix].ay;

               // points[iy][ix].vx *= drag;
               // points[iy][ix].vy *= drag;

                points[iy][ix].x += time_diff * points[iy][ix].vx;
                points[iy][ix].y += time_diff * points[iy][ix].vy;
            }


            clip(points[iy][ix]);
            

          
        }
    }
    
    
    static double t = 0;

    if (force_enable)
    {
        t += force_freq;
    
        double mag = 0.5; // 0.3
    
        int tx = 0;//points_x / 2;
        int ty = points_y / 2;
        
     //    points[ty][tx].x = mag * cos(t) + (((tx+1) / (double)(points_x + 1)) * sheet_x_size + sheet_x_pos);;
         points[ty][tx].y = mag * sin(t) + (((ty+1) / (double)(points_y + 1)) * sheet_y_size + sheet_y_pos);;
    }
    
    
   /* points[0][0].y = max_y / 2.0;
    points[0][0].x = 0;
    
    points[0][1].y = max_y / 2.0 - 1.0;
    points[0][1].x = 0;
    
    
    points[points_y-1][0].y = max_y / 2.0;
    points[points_y-1][0].x = max_x;
    
    points[points_y-1][1].y = max_y / 2.0 - 1.0;
    points[points_y-1][1].x = max_x;;*/
    
    
}
/* End of move objects */



/* World X to screen X */
int screen_x(double x)
{
   return((int)((x/max_x) * display_x));
}
/* End of screen_x */


/* World Y to screen Y */
int screen_y(double y)
{
   return((int)((y/max_y) * display_y));
}
/* End of screen_y */

/* Set pen color */
void set_pen (HPEN &p, HPEN &old_pen, double tension)
{
   // double diff = max_tension ;
  //  if (diff < 0.5) diff = 0.5;

  //  if (max_tension <= 0) max_tension = 0.0001;


    double t = 10.5 * fabs(tension) / spring_k;


//    tension = 0.5 * fabs(tension) / spring_k;

    if (t > 1) t = 1;
    if (t < 0) t = 0;

    int c = (int)(192 * t);// + 63;
  
    if (tension > 0)
        p = CreatePen(PS_SOLID, 2, RGB(c, 0, 0));
    else
        p = CreatePen(PS_SOLID, 2, RGB(0, 0, c));    



    old_pen =  (HPEN ) SelectObject(memdc, p);
                        
    SelectObject(memdc, p);
}


void del_pen (HPEN p, HPEN old_pen)
{
    SelectObject(memdc, old_pen);
    DeleteObject(p);
}






void draw_tensions( void )
{
    
    HPEN pen, old_pen;
    
    // draw horz tensions
    for (int iy = 0; iy < points_y; iy++)
    {
        double x = sheet_x_pos;        
        double y = ((iy+1) / (double)(points_y + 1)) * sheet_y_size + sheet_y_pos;
        
        MoveToEx(memdc, screen_x(points[iy][0].x),  screen_y(points[iy][0].y), NULL);
    
        for(int ix = 1; ix < points_x; ix++)
        {
            set_pen(pen, old_pen, horz_tensions[iy][ix].mag);
        
            LineTo(memdc, screen_x(points[iy][ix].x),   screen_y(points[iy][ix].y));
            
            del_pen(pen, old_pen);            
        }


      //  set_pen(pen, old_pen, horz_tensions[iy][points_x].mag);
      //  LineTo(memdc, screen_x(sheet_x_pos+sheet_x_size), screen_y(y));
       // del_pen(pen, old_pen);                    
    }
    
    // draw vert tensions
    for (int ix = 0; ix < points_x; ix++)
    {
        double x = ((ix+1) / (double)(points_x + 1)) * sheet_x_size + sheet_x_pos;      
        double y = sheet_y_pos;  
        
        MoveToEx(memdc, screen_x(points[0][ix].x),  screen_y(points[0][ix].y), NULL);
    
        for(int iy = 1; iy < points_y; iy++)
        {
            set_pen(pen, old_pen, vert_tensions[iy][ix].mag);

            LineTo(memdc, screen_x(points[iy][ix].x),   screen_y(points[iy][ix].y));

            del_pen(pen, old_pen);
        }

      //  set_pen(pen, old_pen, vert_tensions[points_y][ix].mag);
       // LineTo(memdc, screen_x(x), screen_y(sheet_y_pos+sheet_y_size));
      //  del_pen(pen, old_pen);        
        
    }
    

       // draw vert tensions
    for (int ix = 0; ix < points_x-1; ix++)
    {
//        double x = ((ix+1) / (double)(points_x + 1)) * sheet_x_size + sheet_x_pos;      
//        double y = sheet_y_pos;  
        
   
        for(int iy = 0; iy < points_y-1; iy++)
        {
            set_pen(pen, old_pen, diag1_tensions[iy][ix].mag);

            // draw upperleft  to lower right diagnal
            MoveToEx(memdc,screen_x(points[iy][ix].x), screen_y(points[iy][ix].y), NULL);    
            LineTo(memdc, screen_x(points[iy+1][ix+1].x), screen_y(points[iy+1][ix+1].y));

            del_pen(pen, old_pen);
            
            set_pen(pen, old_pen, diag2_tensions[iy][ix].mag);

            // draw lowerleft  to upper right diagnal
            MoveToEx(memdc,screen_x(points[iy+1][ix].x), screen_y(points[iy+1][ix].y), NULL);    
            LineTo(memdc, screen_x(points[iy][ix+1].x), screen_y(points[iy][ix+1].y));

            del_pen(pen, old_pen);
        }

//        set_pen(pen, old_pen, vert_tensions[points_y][ix].mag);
//        LineTo(memdc, screen_x(x), screen_y(sheet_y_pos+sheet_y_size));
//        del_pen(pen, old_pen);        
        
    } 
    
    
    
    
    
}


void draw_points( void )
{
    for (int y = 0; y < points_y; y++)
        for (int x = 0; x < points_x; x++)
        {
            RECT r;
        
            r.top = screen_y(points[y][x].y - point_size_y);
            r.bottom = screen_y(points[y][x].y + point_size_y);
            
            r.left = screen_x(points[y][x].x - point_size_x);
            r.right = screen_x(points[y][x].x + point_size_x);

            Ellipse(memdc, r.right, r.top, r.left, r.bottom);           
        }
}




/* Draw game display */
void draw( void )
{
    RECT r;
    
    // Erase in black
    SelectObject(memdc, (HBRUSH) GetStockObject(WHITE_BRUSH));
    SelectObject(memdc, (HBRUSH) GetStockObject(BLACK_PEN));    
   
    Rectangle(memdc, 0, 0, display_x, display_y);    
    
    // draw sheet frame
    
  /*  SelectObject(memdc, (HBRUSH) GetStockObject(BLACK_PEN));
      
    MoveToEx(memdc, screen_x(sheet_x_pos),  screen_y(sheet_y_pos), NULL);
       
    LineTo(memdc, screen_x(sheet_x_pos+sheet_x_size),  screen_y(sheet_y_pos));        
    LineTo(memdc, screen_x(sheet_x_pos+sheet_x_size),  screen_y(sheet_y_pos+sheet_y_size));    
    LineTo(memdc, screen_x(sheet_x_pos),  screen_y(sheet_y_pos+sheet_y_size));    
    LineTo(memdc, screen_x(sheet_x_pos),  screen_y(sheet_y_pos));
    */
    draw_tensions();
  
    draw_points();    
 
    char buff[300];
    
    sprintf(buff, "FPS: %d  Points: (%d, %d)  Mass: %8.6f  K: %8.6f  Drag: %8.6f", 
            fps, points_x, points_y, sheet_mass, base_spring_k, drag);


    TextOut(memdc, 2, 2, buff, strlen(buff));
}
/* End of draw_game */


/* determine which point is close tot he passed world cords */
void get_close_point(double wx, double wy, int &ix, int &iy)
{
    double dist = max_x * max_y;
    ix = 1;
    iy = 0;
    
    for (int y = 0; y < points_y; y++)
        for (int x = 0; x < points_x; x++)
        {
            double d = pow(wx - points[y][x].x, 2) + 
                        pow(wy - points[y][x].y, 2);
                        
            if (d < dist)
            {
                dist =d;
                ix = x;
                iy = y;
            }
    
    }    
}
/* End of get_close_point */


/* Create off screen buffer */
void setup_display( HWND hwnd )
{
    // Get main DC
    HDC hdc = GetDC(hwnd);

  	// Create Memory device contextand bitmap
    memdc = CreateCompatibleDC(hdc);	

    // Create off screen buffer
  	h_mem_bitmap = CreateCompatibleBitmap(hdc, display_x, display_y);

  	// Select the off screen buffer into the DC
   	SelectObject(memdc, h_mem_bitmap);		

    ReleaseDC(hwnd, hdc);       // Main DC is no longer needed
}
/* End of setup_display */



/* Resize the display and update buffers */
void resize_display(HWND hwnd, LPARAM l)
{
    display_x = LOWORD(l);
    display_y = HIWORD(l);

    DeleteObject(h_mem_bitmap);		//	 Delete old screen buffer
    DeleteDC(memdc);		      	// Delete old memory DC

    setup_display(hwnd);

    InvalidateRect(hwnd, NULL, true);   
}
/* End of resize_display */



/* The window Function */
LRESULT CALLBACK WinProc(HWND hwnd, UINT msg, WPARAM w, LPARAM l)
{
    HDC hdc;

    switch(msg)
    {
        case WM_CREATE: 
        {
            setup_display(hwnd);
          
//            FILE *f = fopen("data.txt", "wt");
//            fclose(f);
                   
            init_sheet( );          
 
            SetTimer(hwnd, UPDATE_TIMER, FRAME_TIME, NULL);
            SetTimer(hwnd, SECOND_TIMER, 1000, NULL);
            return(0);
        }

        case WM_CHAR:
        {
            if (w == 27) DestroyWindow(hwnd);   // Escape exits

            if (w == 'i') init_sheet(); 
            if (w == 'z')  apply_grav = !apply_grav;
            

            if (w == 'q' && points_y < max_points_y-1) {points_y++; init_sheet(); }
            if (w == 'a' && points_y > 1) {points_y--; init_sheet(); }

            if (w == 'w' && points_x < max_points_x-1) {points_x++; init_sheet(); }
            if (w == 's' && points_x > 1) {points_x--; init_sheet(); }

            if (w == 'x') force_enable = !force_enable;

            if (w == 'e'){ sheet_mass *= 1.01; update_const(); }
            if (w == 'd') {sheet_mass /= 1.01; update_const(); }

            if (w == 'r'){ base_spring_k *= 1.01; update_const(); }
            if (w == 'f'){ base_spring_k /= 1.01; update_const(); }

            if (w == 't') drag *= 1.01; 
            if (w == 'g') drag /= 1.01; 

            return(0);
        }

        case WM_PAINT:
            PAINTSTRUCT ps;
            hdc = BeginPaint(hwnd, &ps);
            BitBlt(hdc, 0, 0, display_x, display_y, memdc, 0, 0, SRCCOPY);
            EndPaint(hwnd, &ps);  
           return(0);

        case WM_TIMER:            
            if (w == UPDATE_TIMER)
            {
                calc_tension();

                update_points(hwnd);
                draw();
                updates++;
                

                // Force redraw 
                InvalidateRect(hwnd, NULL, 0);                
            }
            if (w == SECOND_TIMER)
            {               
                fps = updates;
                updates = 0;
            }
            return(0);  
            
        case WM_LBUTTONDOWN:
        {
            if (w & MK_CONTROL)                    init_sheet();
            
          
            
            inital_mouse_x = (double)(LOWORD(l) / (float)display_x) * max_x;
            inital_mouse_y = (double)(HIWORD(l) / (float)display_y) * max_y;

            get_close_point(inital_mouse_x, inital_mouse_y, test_pt_x, test_pt_y);

            mouse_down = true;
            points[test_pt_y][test_pt_x].x = inital_mouse_x;
            points[test_pt_y][test_pt_x].y = inital_mouse_y;

            mouse_time = get_time();
            //reset_ball();
        }
            
        return(0);
    
        case WM_LBUTTONUP:
        {                
            double x = (double)(LOWORD(l) / (float)display_x) * max_x;
            double y = (double)(HIWORD(l) / (float)display_y) * max_y;
                
            mouse_down = false;

            double time_diff = get_time() - mouse_time;

            if (time_diff < 0.0001) time_diff = 0.0001;
            
            double dist = sqrt((x - inital_mouse_x) * (x - inital_mouse_x) + 
                               (y - inital_mouse_y) * (y - inital_mouse_y));

            double angle = atan2(y - inital_mouse_y, x - inital_mouse_x);

            points[test_pt_y][test_pt_x].vx = 1.5 * dist * cos(angle) / time_diff;
            points[test_pt_y][test_pt_x].vy = 1.5 * dist * sin(angle) / time_diff;
        }
        return(0);       
                 
        case WM_RBUTTONDOWN:
            return(0);

        case WM_DESTROY:    
            DeleteObject(h_mem_bitmap);		//	 Delete off screen buffer
            DeleteDC(memdc);	       		// Delete memory DC
        
            PostQuitMessage(0);
            return(0);
            
        case WM_MOUSEMOVE:
            if (mouse_down)
            {
             
             points[test_pt_y][test_pt_x].x = (double)(LOWORD(l) / (float)display_x) * max_x;
             points[test_pt_y][test_pt_x].y = (double)(HIWORD(l) / (float)display_y) * max_y;
            }
            
            return(0);
            
            
         case WM_KILLFOCUS:
            mouse_down = 0;
            return(0);            
            
            
        case WM_SIZE:
            resize_display(hwnd, l);
            return(0);

        default:
            return(DefWindowProc(hwnd, msg, w, l));
    }

    return(0);  
}
/* End of WinProc */


/* WinMain */
int WINAPI WinMain(HINSTANCE h_this_inst, HINSTANCE h_prev_inst, LPSTR args, int mode)
{
    WNDCLASS wnd_class;

    char program_name[] = "SMASS";
        
    wnd_class.hInstance = h_this_inst;
    wnd_class.lpszClassName = program_name;
    wnd_class.lpfnWndProc = WinProc;
    wnd_class.style = CS_OWNDC;//0;
    
    wnd_class.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wnd_class.hCursor = LoadCursor(NULL, IDC_ARROW);
    wnd_class.lpszMenuName = NULL;
    
    wnd_class.cbClsExtra = 0;
    wnd_class.cbWndExtra = 0;
    
    wnd_class.hbrBackground = NULL; // Not needed
    
    if (!RegisterClass(&wnd_class)) return(0);
        
    // Create the window
    HWND hwnd = CreateWindow(program_name, "Spring Mass system - 2007 Jason Hunt", 
                             WS_OVERLAPPEDWINDOW,
                             300, 100,                                      // Pos
                             display_x, display_y,              // Size
                             HWND_DESKTOP, NULL, h_this_inst, NULL);

    ShowWindow(hwnd, mode);
    UpdateWindow(hwnd);
           
    MSG msg;
    
    // Windows message loop
    while(GetMessage(&msg, NULL, 0, 0))
    { 
        TranslateMessage(&msg);
        DispatchMessage(&msg);                            
    }
                        
    return(msg.wParam);
}
/* End of winmain */



